{
  description = "horizon-spec";

  inputs = {
    flake-utils.url = "github:numtide/flake-utils";
    horizon-platform.url = "git+https://gitlab.homotopic.tech/horizon/horizon-platform";
    lint-utils = {
      url = "git+https://gitlab.homotopic.tech/nix/lint-utils";
      inputs.nixpkgs.follows = "nixpkgs";
    };
    nixpkgs.url = "github:nixos/nixpkgs/nixpkgs-unstable";
  };

  outputs =
    inputs@
    { self
    , flake-utils
    , horizon-platform
    , lint-utils
    , nixpkgs
    , ...
    }:
    flake-utils.lib.eachSystem [ "x86_64-linux" ]
      (system:
      let
        pkgs = import nixpkgs { inherit system; };
      in
      with pkgs.dhallPackages;
      with pkgs.haskell.lib.compose;
      with pkgs.stdenv;
      with pkgs.writers;
      let

        dhall-prelude = buildDhallGitHubPackage {
          name = "Prelude";
          owner = "dhall-lang";
          repo = "dhall-lang";
          directory = "Prelude";
          file = "package.dhall";
          rev = "v21.1.0";
          sha256 = "sha256-7bIKrdGrlq4NmaFWYWz5Ov2DhslxunoAdUNSMtKGglY=";
          source = true;
        };

        dhall-horizon-spec = buildDhallPackage {
          name = "horizon-spec";
          dependencies = [ dhall-prelude ];
          code = "${./.}/dhall/package.dhall";
        };

        overrides = hfinal: hprev:
          {
            horizon-spec = dontCheck (disableLibraryProfiling (hprev.callCabal2nix "horizon-spec" ./. { }));
          };

        legacyPackages = horizon-platform.legacyPackages.${system}.extend overrides;

        horizon-spec-tests-wrapper = writeBashBin "horizon-spec-tests-wrapper" ''
          DHALL_PRELUDE=${dhall-prelude}/source.dhall ${legacyPackages.horizon-spec}/bin/horizon-spec-tests
        '';

      in
      {

        checks =
          with lint-utils.outputs.linters.${system}; {
            cabal-fmt = cabal-fmt { src = self; };
            dhall-format = dhall-format { src = self; };
            hlint = hlint { src = self; };
            horizon-spec-tee-check = tee-check {
              name = "horizon-spec-tee-check";
              src = self;
              exe = "${horizon-spec-tests-wrapper}/bin/horizon-spec-tests-wrapper";
            };
            nixpkgs-fmt = nixpkgs-fmt { src = self; };
            stylish-haskell = stylish-haskell { src = self; };
            werror = werror { pkg = legacyPackages.horizon-spec; };
          };

        devShells.default = legacyPackages.horizon-spec.env.overrideAttrs (attrs: {
          buildInputs = attrs.buildInputs ++ [
            legacyPackages.cabal-install
            pkgs.nixpkgs-fmt
          ];
        });

        inherit legacyPackages;

        packages = {
          default = legacyPackages.horizon-spec;
          dhall-horizon-spec = dhall-horizon-spec;
        };

      });
}
